import numpy
from distutils.core import setup
from distutils.extension import Extension

try:
    from Cython.Distutils import build_ext
except ImportError:
    use_cython = False
else:
    use_cython = True


cmdclass = {}
ext_modules = []


if use_cython:
    
    ext_modules += [Extension("adaoptc", 
                ["adaoptc.pyx"],
                libraries=["m"],
                extra_compile_args=["-ffast-math", "-fopenmp"],
                extra_link_args=["-fopenmp"],
                include_dirs=[numpy.get_include()])]
    cmdclass.update({'build_ext': build_ext})
    
else:
    
    ext_modules += [Extension("adaoptc", 
                ["adaoptc.c"],
                libraries=["m"],
                extra_compile_args=["-ffast-math", "-fopenmp"],
                extra_link_args=["-fopenmp"],
                include_dirs=[numpy.get_include()])]

    
setup(name="adaoptc",
      cmdclass=cmdclass,
      ext_modules=ext_modules)



import numpy as np
import copy 
from tqdm import tqdm 

        

# from https://www.kaggle.com/arjanso/reducing-dataframe-memory-size-by-65

def reduce_mem_usage(df, verbose=True, copy_input=True):
    
    if copy_input is True:
        df_res = copy.deepcopy(df)
    else:
        df_res = df
    
    numerics = ['int16', 'int32', 'int64', 'float16', 'float32', 'float64']
    start_mem = df_res.memory_usage().sum() / 1024**2    
    
    
    for idx, col in tqdm(enumerate(df_res.columns)):
        col_type = df_res[col].dtypes
        if col_type in numerics:
            c_min = df_res[col].min()
            c_max = df_res[col].max()
            if str(col_type)[:3] == 'int':
                if c_min > np.iinfo(np.int8).min and c_max < np.iinfo(np.int8).max:
                    df_res[col] = df_res[col].astype(np.int8)
                elif c_min > np.iinfo(np.int16).min and c_max < np.iinfo(np.int16).max:
                    df_res[col] = df_res[col].astype(np.int16)
                elif c_min > np.iinfo(np.int32).min and c_max < np.iinfo(np.int32).max:
                    df_res[col] = df_res[col].astype(np.int32)
                elif c_min > np.iinfo(np.int64).min and c_max < np.iinfo(np.int64).max:
                    df_res[col] = df_res[col].astype(np.int64)  
            else:
                if c_min > np.finfo(np.float16).min and c_max < np.finfo(np.float16).max:
                    df_res[col] = df_res[col].astype(np.float16)
                elif c_min > np.finfo(np.float32).min and c_max < np.finfo(np.float32).max:
                    df_res[col] = df_res[col].astype(np.float32)
                else:
                    df_res[col] = df_res[col].astype(np.float64)    
    
    pbar.update(len(df_res.columns))
    
    end_mem = df_res.memory_usage().sum() / 1024**2
    if verbose: print('Mem. usage decreased to {:5.2f} Mb ({:.1f}% reduction)'.format(end_mem, 100 * (start_mem - end_mem) / start_mem))
    return df_res


from .sampling.rowsubsampling import subsample
from .misc.misc import merge_two_dicts, flatten, is_float, is_factor
from .memoryuse import reduce_mem_usage


__all__ = [subsample, 
           merge_two_dicts, flatten, is_float, is_factor, 
           reduce_mem_usage]



import numpy as np
import pickle
from sklearn.base import BaseEstimator
from sklearn.base import ClassifierMixin
from ..utils import subsample
from ..adaopt_cython import fit_adaopt, predict_proba_adaopt



class AdaOpt(BaseEstimator, ClassifierMixin):
    """AdaOpt classifier
        
       Attributes
       ----------
       n_iterations: int
           number of iterations of the optimizer at training time
       learning_rate: float
           controls the speed of the optimizer at training time  
       reg_lambda: float
           L2 regularization parameter for successive errors in the optimizer 
           (at training time)
       reg_alpha: float
           L1 regularization parameter for successive errors in the optimizer 
           (at training time)
       eta: float
           controls the slope in gradient descent (at training time)
       gamma: float
           controls the step size in gradient descent (at training time)
       k: int
           number of nearest neighbors selected at test time for classification
       tolerance: float
           controls early stopping in gradient descent (at training time)  
       n_clusters: int
            number of clusters, if MiniBatch k-means is used at test time 
            (for faster prediction)
       batch_size: int
            size of the batch, if MiniBatch k-means is used at test time 
            (for faster prediction)
       row_sample: float
           percentage of rows chosen from training set (by stratified subsampling, 
           for faster prediction)
       type_dist: str
           distance used for finding the nearest neighbors; currently `euclidean-f`
           (euclidean distances calculated as whole), `euclidean` (euclidean distances 
           calculated row by row), `cosine` (cosine distance)
       cache: boolean
           if the nearest neighbors are cached or not, for faster retrieval in 
           subsequent calls 
       seed: int 
           reproducibility seed for nodes_sim=='uniform', clustering and dropout
    """
    
    def __init__(
        self,
        n_iterations=50,
        learning_rate=0.3,
        reg_lambda=0.1,
        reg_alpha=0.5,
        eta=0.01,
        gamma=0.01,
        k=3,
        tolerance=0,
        n_clusters=0,
        batch_size=100,
        row_sample=0.8, 
        type_dist="euclidean-f",
        cache=True,
        seed=123,
    ):

        assert type_dist in (
            "euclidean",
            "manhattan",
            "euclidean-f",
            "cosine",
        ), "must have: `type_dist` in ('euclidean', 'manhattan', 'euclidean-f', 'cosine') "

        self.n_iterations = n_iterations
        self.learning_rate = learning_rate
        self.reg_lambda = reg_lambda
        self.reg_alpha = reg_alpha
        self.eta = eta
        self.gamma = gamma
        self.k = k
        self.tolerance = tolerance
        self.n_clusters = n_clusters
        self.batch_size = batch_size
        self.row_sample = row_sample        
        self.type_dist = type_dist
        self.cache = cache
        self.seed = seed


    def fit(self, X, y, **kwargs):
        """Fit AdaOpt to training data (X, y)
        
        Parameters
        ----------
        X: {array-like}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number 
            of samples and n_features is the number of features
        
        y: array-like, shape = [n_samples]
               Target values
    
        **kwargs: additional parameters to be passed to self.cook_training_set
               
        Returns
        -------
        self: object
        """
        
        if self.row_sample < 1:
            index_subsample = subsample(y, row_sample=self.row_sample, 
                                       seed=self.seed)
            y_ = y[index_subsample]
            X_ = X[index_subsample, :]
        else:
            y_ = pickle.loads(pickle.dumps(y, -1))
            X_ = pickle.loads(pickle.dumps(X, -1))
        
        n, p = X_.shape
            
        n_classes = len(np.unique(y_))

        assert n == len(y_), "must have X.shape[0] == len(y)"

        res = fit_adaopt(
            X=np.asarray(X_),
            y=np.asarray(y_),
            n_iterations=self.n_iterations,
            n_X=n,
            p_X=p,
            n_classes=n_classes,
            learning_rate=self.learning_rate,
            reg_lambda=self.reg_lambda,
            reg_alpha=self.reg_alpha,
            eta=self.eta,
            gamma=self.gamma,
            tolerance=self.tolerance,
        )

        self.probs_training = res["probs"]
        self.training_accuracy = res["training_accuracy"]
        self.alphas = res["alphas"]
        self.n_iterations = res["n_iterations"]
        self.scaled_X_train = res["scaled_X_train"]

        return self


    def predict(self, X, **kwargs):
        """Predict test data X.
        
        Parameters
        ----------
        X: {array-like}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number 
            of samples and n_features is the number of features.
        
        **kwargs: additional parameters to be passed to `predict_proba`
                
               
        Returns
        -------
        model predictions: {array-like}
        """

        return np.argmax(self.predict_proba(X, **kwargs), 
                         axis=1)


    def predict_proba(self, X, **kwargs):
        """Predict probabilities for test data X.
        
        Parameters
        ----------
        X: {array-like}, shape = [n_samples, n_features]
            Training vectors, where n_samples is the number 
            of samples and n_features is the number of features.
        
        **kwargs: additional parameters to be passed to 
                  self.cook_test_set
               
        Returns
        -------
        probability estimates for test data: {array-like}        
        """

        n_train = self.scaled_X_train.shape[0]

        n_test = X.shape[0]
        
        return predict_proba_adaopt(X_test=np.asarray(X, order='C'), 
                                scaled_X_train=np.asarray(self.scaled_X_train, order='C'),
                                n_test=n_test, n_train=n_train,
                                probs_train=self.probs_training,
                                k=self.k, n_clusters=self.n_clusters,
                                batch_size=self.batch_size, 
                                type_dist=self.type_dist, 
                                cache=self.cache,
                                seed=self.seed)



"""Top-level package for mlsauce."""

__author__ = """T. Moudiki"""
__email__ = "thierry.moudiki@gmail.com"
__version__ = "__version__ = '0.3.0'"


from .adaopt_cython.adaoptc import fit_adaopt, predict_proba_adaopt
from .adaopt.adaopt import AdaOpt


__all__ = ["AdaOpt", "fit_adaopt", "predict_proba_adaopt"]



#!/usr/bin/env python

from setuptools import dist, setup, find_packages

dist.Distribution().fetch_build_eggs(["Cython >= 0.29.13", "numpy >= 1.13.0"])

import numpy
from setuptools.extension import Extension



with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

    
requirements = ["Cython >= 0.29.13", 
                "numpy >= 1.13.0", "scipy >= 0.19.0", 
                "scikit-learn >= 0.18.0", "tqdm>=4.46.0"]


setup_requirements = [ ]

test_requirements = [ ]    


extensions = [
    Extension(name="mlsauce.adaopt_cython.adaoptc", 
              sources=[ "mlsauce/adaopt_cython/adaoptc.c" ],
              include_dirs = [numpy.get_include()],
              libraries=["m"],
              extra_compile_args=["-ffast-math", "-fopenmp"], 
              extra_link_args=["-fopenmp"]),
]


setup(
    author="T. Moudiki",
    author_email='thierry.moudiki@gmail.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="Miscellaneous Statistical/Machine Learning tools",
    install_requires=requirements,
    license="BSD3 Clear license",
    long_description="Miscellaneous Statistical/Machine Learning stuff",
    include_package_data=True,
    ext_modules=extensions, # cython modules
    keywords='mlsauce',
    name='mlsauce',
    packages=find_packages(include=['mlsauce', 'mlsauce.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/thierrymoudiki/mlsauce',
    version='0.3.0',
    zip_safe=False,
)
